@@ -5,7 +5,6 @@ use Symbol;
 use FileHandle;
 use Exporter;
 use bytes;
-use Scalar::Util qw( weaken );
 
 use 5.000;
 
@@ -13,7 +12,7 @@ use vars qw( @ISA $VERSION $AUTOLOAD @EXPORT @EXPORT_OK );
 
 @ISA = qw( Exporter FileHandle );
 
-$VERSION = '0.16.1';
+$VERSION = '0.10';
 
 @EXPORT = @FileHandle::EXPORT;
 @EXPORT_OK = @FileHandle::EXPORT_OK;
@@ -87,7 +86,7 @@ sub new
 
   my $self;
 
-  if (defined $_[0] && defined fileno $_[0])
+  if (defined $_[0] && UNIVERSAL::isa($_[0],'IO::Handle'))
   {
     $self = shift;
   }
@@ -97,16 +96,9 @@ sub new
     return undef unless defined $self;
   }
 
-  my $values =
-    {
-      'fh' => $self,
-      'eof_called' => 0,
-      'filehandle_unget_buffer' => '',
-    };
+  tie *$self, "${class}::Tie", $self;
 
-  weaken($values->{'fh'});
-  
-  tie *$self, "${class}::Tie", $values;
+  ${*$self}{'filehandle_unget_buffer'} = '';
 
   bless $self, $class;
   return $self;
@@ -114,35 +106,11 @@ sub new
 
 #-------------------------------------------------------------------------------
 
-sub new_from_fd
+sub read
 {
-  my $class = shift;
+  my $self = shift;
 
-  my $self;
-
-#  if (defined $_[0] && defined fileno $_[0])
-#  {
-#    $self = shift;
-#  }
-#  else
-  {
-    $self = $class->SUPER::new_from_fd(@_);
-    return undef unless defined $self;
-  }
-
-  my $values =
-    {
-      'fh' => $self,
-      'eof_called' => 0,
-      'filehandle_unget_buffer' => '',
-    };
-
-  weaken($values->{'fh'});
-  
-  tie *$self, "${class}::Tie", $values;
-
-  bless $self, $class;
-  return $self;
+  tied(*$self)->read(@_);
 }
 
 #-------------------------------------------------------------------------------
@@ -152,7 +120,7 @@ sub ungetc
   my $self = shift;
   my $ord = shift;
 
-  substr(tied(*$self)->{'filehandle_unget_buffer'},0,0) = chr($ord);
+  substr(${*$self}{'filehandle_unget_buffer'},0,0) = chr($ord);
 }
 
 #-------------------------------------------------------------------------------
@@ -162,7 +130,7 @@ sub ungets
   my $self = shift;
   my $string = shift;
 
-  substr(tied(*$self)->{'filehandle_unget_buffer'},0,0) = $string;
+  substr(${*$self}{'filehandle_unget_buffer'},0,0) = $string;
 }
 
 #-------------------------------------------------------------------------------
@@ -171,33 +139,8 @@ sub buffer
 {
   my $self = shift;
 
-  tied(*$self)->{'filehandle_unget_buffer'} = shift if @_;
-  return tied(*$self)->{'filehandle_unget_buffer'};
-}
-
-#-------------------------------------------------------------------------------
-
-sub input_record_separator
-{
-  my $self = shift;
-
-  if(@_)
-  {
-    tied(*$self)->{'input_record_separator'} = shift;
-    tied(*$self)->{'input_record_separator_specified'} = 1;
-  }
-
-  return undef unless exists tied(*$self)->{'input_record_separator'};
-  return tied(*$self)->{'input_record_separator'};
-}
-
-#-------------------------------------------------------------------------------
-
-sub clear_input_record_separator
-{
-  my $self = shift;
-
-  delete tied(*$self)->{'input_record_separator'};
+  ${*$self}{'filehandle_unget_buffer'} = shift if @_;
+  return ${*$self}{'filehandle_unget_buffer'};
 }
 
 ###############################################################################
@@ -221,9 +164,9 @@ $VERSION = '0.10';
 
 my %tie_mapping = (
   PRINT => 'print', PRINTF => 'printf', WRITE => 'syswrite',
-  READLINE => 'getline_wrapper', GETC => 'getc', READ => 'read', CLOSE => 'close',
+  READLINE => 'getline', GETC => 'getc', READ => 'read', CLOSE => 'close',
   BINMODE => 'binmode', OPEN => 'open', EOF => 'eof', FILENO => 'fileno',
-  SEEK => 'seek', TELL => 'tell', FETCH => 'fetch',
+  SEEK => 'seek', TELL => 'tell',
 );
 
 #-------------------------------------------------------------------------------
@@ -243,6 +186,8 @@ sub AUTOLOAD
     {
       my $self = shift;
 
+      $sub = 'getlines' if $sub eq 'getline' && wantarray;
+
       if (defined &$sub)
       {
         &$sub($self,@_);
@@ -260,7 +205,7 @@ sub AUTOLOAD
 
         $self->{'fh'}->$sub(@_);
 
-        tie *{$self->{'fh'}}, __PACKAGE__, $self;
+        tie *{$self->{'fh'}}, __PACKAGE__, $self->{'fh'};
       }
     };
 
@@ -279,10 +224,13 @@ sub DESTROY
 sub TIEHANDLE
 {
   my $class = shift;
-  my $self = shift;
 
-  bless($self, $class);
+  my $self;
 
+  $self = bless({}, $class);
+
+  $self->{'fh'} = $_[0];
+  
   return $self;
 }
 
@@ -292,9 +240,6 @@ sub binmode
 {
   my $self = shift;
 
-  warn "Under windows, calling binmode after eof exposes a bug that exists in some versions of Perl.\n"
-    if $self->{'eof_called'};
-
   # Prevent recursion
   # Temporarily disable warnings so that we don't get "untie attempted
   # while 1 inner references still exist". Not sure what's the "right
@@ -313,7 +258,7 @@ sub binmode
     binmode $self->{'fh'};
   }
 
-  tie *{$self->{'fh'}}, __PACKAGE__, $self;
+  tie *{$self->{'fh'}}, __PACKAGE__, $self->{'fh'};
 }
 
 #-------------------------------------------------------------------------------
@@ -333,27 +278,13 @@ sub fileno
 
   my $fileno = fileno $self->{'fh'};
 
-  tie *{$self->{'fh'}}, __PACKAGE__, $self;
+  tie *{$self->{'fh'}}, __PACKAGE__, $self->{'fh'};
 
   return $fileno;
 }
 
 #-------------------------------------------------------------------------------
 
-sub getline_wrapper
-{
-  if (wantarray)
-  {
-    goto &getlines;
-  }
-  else
-  {
-    goto &getline;
-  }
-}
-
-#-------------------------------------------------------------------------------
-
 sub getline
 {
   my $self = shift;
@@ -369,33 +300,17 @@ sub getline
 
   my $line;
 
-  my $old_input_record_separator = $INPUT_RECORD_SEPARATOR;
-  $INPUT_RECORD_SEPARATOR = $self->{'input_record_separator'}
-    if exists $self->{'input_record_separator'};
-  if (defined $INPUT_RECORD_SEPARATOR &&
-      $self->{'filehandle_unget_buffer'} =~ /(.*?$INPUT_RECORD_SEPARATOR)/)
+  if (${*{$self->{'fh'}}}{'filehandle_unget_buffer'} =~ /(.*?$INPUT_RECORD_SEPARATOR)/)
   {
     $line = $1;
-    substr($self->{'filehandle_unget_buffer'},0,length $line) = '';
+    substr(${*{$self->{'fh'}}}{'filehandle_unget_buffer'},0,length $line) = '';
   }
   else
   {
-    $line = $self->{'filehandle_unget_buffer'};
-    $self->{'filehandle_unget_buffer'} = '';
-    my $templine = $self->{'fh'}->getline(@_);
-
-    if ($line eq '' && !defined $templine)
-    {
-      $line = undef;
-    }
-    else
-    {
-      $line .= $templine;
-    }
+    $line = $self->{'fh'}->getline(@_);
   }
-  $INPUT_RECORD_SEPARATOR = $old_input_record_separator;
 
-  tie *{$self->{'fh'}}, __PACKAGE__, $self;
+  tie *{$self->{'fh'}}, __PACKAGE__, $self->{'fh'};
 
   return $line;
 }
@@ -416,53 +331,25 @@ sub getlines
   }
 
   my @buffer_lines;
+  ${*{$self->{'fh'}}}{'filehandle_unget_buffer'} =~
+    s/^(.*$INPUT_RECORD_SEPARATOR)/push @buffer_lines, $1;''/mge;
 
-  my $old_input_record_separator = $INPUT_RECORD_SEPARATOR;
-  $INPUT_RECORD_SEPARATOR = $self->{'input_record_separator'}
-    if exists $self->{'input_record_separator'};
-  if (defined $INPUT_RECORD_SEPARATOR)
+  my @other_lines = $self->{'fh'}->getlines(@_);
+
+  if (@other_lines)
   {
-    $self->{'filehandle_unget_buffer'} =~
-      s/^(.*$INPUT_RECORD_SEPARATOR)/push @buffer_lines, $1;''/mge;
-
-    my @other_lines = $self->{'fh'}->getlines(@_);
-
-    if (@other_lines)
-    {
-      if (defined $other_lines[0])
-      {
-        substr($other_lines[0],0,0) = $self->{'filehandle_unget_buffer'};
-      }
-    }
-    else
-    {
-      if ($self->{'filehandle_unget_buffer'} ne '')
-      {
-        unshift @other_lines, $self->{'filehandle_unget_buffer'};
-      }
-    }
-
-    $self->{'filehandle_unget_buffer'} = '';
-
-    push @buffer_lines, @other_lines;
+    substr($other_lines[0],0,0) = ${*{$self->{'fh'}}}{'filehandle_unget_buffer'};
   }
   else
   {
-    $buffer_lines[0] = $self->{'filehandle_unget_buffer'};
-    my $templine = ($self->{'fh'}->getlines(@_))[0];
-
-    if ($buffer_lines[0] eq '' && !defined $templine)
-    {
-      $buffer_lines[0] = undef;
-    }
-    else
-    {
-      $buffer_lines[0] .= $templine;
-    }
+    unshift @other_lines, ${*{$self->{'fh'}}}{'filehandle_unget_buffer'};
   }
-  $INPUT_RECORD_SEPARATOR = $old_input_record_separator;
 
-  tie *{$self->{'fh'}}, __PACKAGE__, $self;
+  ${*{$self->{'fh'}}}{'filehandle_unget_buffer'} = '';
+
+  push @buffer_lines, @other_lines;
+
+  tie *{$self->{'fh'}}, __PACKAGE__, $self->{'fh'};
 
   return @buffer_lines;
 }
@@ -484,17 +371,17 @@ sub getc
 
   my $char;
 
-  if ($self->{'filehandle_unget_buffer'} ne '')
+  if (${*{$self->{'fh'}}}{'filehandle_unget_buffer'} ne '')
   {
-    $char = substr($self->{'filehandle_unget_buffer'},0,1);
-    substr($self->{'filehandle_unget_buffer'},0,1) = '';
+    $char = substr(${*{$self->{'fh'}}}{'filehandle_unget_buffer'},0,1);
+    substr(${*{$self->{'fh'}}}{'filehandle_unget_buffer'},0,1) = '';
   }
   else
   {
     $char = $self->{'fh'}->getc(@_);
   }
 
-  tie *{$self->{'fh'}}, __PACKAGE__, $self;
+  tie *{$self->{'fh'}}, __PACKAGE__, $self->{'fh'};
 
   return $char;
 }
@@ -520,10 +407,10 @@ sub read
 
   my $num_bytes_read = 0;
 
-  if ($self->{'filehandle_unget_buffer'} ne '')
+  if (${*{$self->{'fh'}}}{'filehandle_unget_buffer'} ne '')
   {
-    my $read_string = substr($self->{'filehandle_unget_buffer'},0,$length);
-    substr($self->{'filehandle_unget_buffer'},0,$length) = '';
+    my $read_string = substr(${*{$self->{'fh'}}}{'filehandle_unget_buffer'},0,$length);
+    substr(${*{$self->{'fh'}}}{'filehandle_unget_buffer'},0,$length) = '';
 
     my $num_bytes_buffer = length $read_string;
 
@@ -558,7 +445,7 @@ sub read
     }
   }
 
-  tie *{$self->{'fh'}}, __PACKAGE__, $self;
+  tie *{$self->{'fh'}}, __PACKAGE__, $self->{'fh'};
 
   return $num_bytes_read;
 }
@@ -580,18 +467,14 @@ sub seek
     untie *{$self->{'fh'}};
   }
 
-  if($whence != 0 && $whence != 1 && $whence != 2)
-  {
-    tie *{$self->{'fh'}}, __PACKAGE__, $self;
-    return 0;
-  }
+  return 0 if $whence != 0 && $whence != 1 && $whence != 2;
 
   my $status;
 
   # First try to seek using the built-in seek
   if (seek($self->{'fh'},$position,$whence))
   {
-    $self->{'filehandle_unget_buffer'} = '';
+    ${*{$self->{'fh'}}}{'filehandle_unget_buffer'} = '';
     $status = 1;
   }
   else
@@ -606,7 +489,7 @@ sub seek
     {
       if ($absolute_position >= $self->tell)
       {
-        substr($self->{'filehandle_unget_buffer'}, 0,
+        substr(${*{$self->{'fh'}}}{'filehandle_unget_buffer'}, 0,
           $absolute_position - $self->tell) = '';
         $status = 1;
       }
@@ -624,7 +507,7 @@ sub seek
     }
   }
 
-  tie *{$self->{'fh'}}, __PACKAGE__, $self;
+  tie *{$self->{'fh'}}, __PACKAGE__, $self->{'fh'};
 
   return $status;
 }
@@ -646,15 +529,11 @@ sub tell
 
   my $file_position = tell $self->{'fh'};
 
-  if ($file_position == -1)
-  {
-    tie *{$self->{'fh'}}, __PACKAGE__, $self;
-    return -1;
-  }
+  return -1 if $file_position == -1;
 
-  $file_position -= length($self->{'filehandle_unget_buffer'});
+  $file_position -= length(${*{$self->{'fh'}}}{'filehandle_unget_buffer'});
 
-  tie *{$self->{'fh'}}, __PACKAGE__, $self;
+  tie *{$self->{'fh'}}, __PACKAGE__, $self->{'fh'};
 
   return $file_position;
 }
@@ -676,7 +555,7 @@ sub eof
 
   my $eof;
 
-  if ($self->{'filehandle_unget_buffer'} ne '')
+  if (${*{$self->{'fh'}}}{'filehandle_unget_buffer'} ne '')
   {
     $eof = 0;
   }
@@ -685,21 +564,11 @@ sub eof
     $eof = $self->{'fh'}->eof();
   }
 
-  tie *{$self->{'fh'}}, __PACKAGE__, $self;
-
-  $self->{'eof_called'} = 1;
+  tie *{$self->{'fh'}}, __PACKAGE__, $self->{'fh'};
 
   return $eof;
 }
 
-#-------------------------------------------------------------------------------
-
-sub fetch
-{
-  my $self = shift;
-  return $self;
-}
-
 1;
 
 __END__
@@ -713,19 +582,19 @@ FileHandle::Unget - FileHandle which supports multi-byte unget
 
 =head1 SYNOPSIS
 
-  use FileHandle::Unget;
-  
-  # open file handle
-  my $fh = FileHandle::Unget->new("file")
-    or die "cannot open filehandle: $!";
-  
-  my $buffer;
-  read($fh,$buffer,100);
-  print $buffer;
+    use FileHandle::Unget;
+    
+    # open file handle
+    my $fh = FileHandle::Unget->new("file")
+      or die "cannot open filehandle: $!";
+    
+    my $buffer;
+    read($fh,$buffer,100);
+    print $buffer;
 
-  print <$fh>;
-  
-  $fh->close;
+    print <$fh>;
+    
+    $fh->close;
 
 
 =head1 DESCRIPTION
@@ -764,7 +633,6 @@ input stream. This method can be called more than once in a row to put
 multiple values back on the stream. Memory usage is equal to the total number
 of bytes pushed back.
 
-
 =item $fh->ungets ( BUF )
 
 Pushes a buffer back onto the given handle's input stream. This method can be
@@ -774,27 +642,10 @@ the stream.  Memory usage is equal to the total number of bytes pushed back.
 The buffer is not processed in any way--managing end-of-line characters and
 whatnot is your responsibility.
 
-
 =item $fh->buffer ( [BUF] )
 
 Get or set the pushback buffer directly.
 
-
-=item $fh->input_record_separator ( STRING )
-
-Get or set the per-filehandle input record separator. After it is called, the
-input record separator for the filehandle is independent of the global $/.
-Until this method is called (and after clear_input_record_separator is called)
-the global $/ is used.
-
-
-=item $fh->clear_input_record_separator ()
-
-Clear the per-filehandle input record separator. This removes the
-per-filehandle input record separator semantics, reverting the filehandle to
-the normal global $/ semantics.
-
-
 =item tell ( $fh )
 
 C<tell> returns the actual file position minus the length of the unget buffer.
@@ -808,7 +659,6 @@ the position indicated by C<tell>.  Second, C<tell> will return a negative
 number if you unget more bytes than you read. (This can be problematic since
 this function returns -1 on error.)
 
-
 =item seek ( $fh, [POSITION], [WHENCE] )
 
 C<seek> defaults to the standard seek if possible, clearing the unget buffer
@@ -831,29 +681,12 @@ To test that this module is indeed a drop-in replacement for FileHandle, the
 following modules were modified to use FileHandle::Unget, and tested using
 "make test". They have all passed.
 
+CPAN-1.76
+
 
 =head1 BUGS
 
-There is a bug in Perl on Windows that is exposed if you open a stream, then
-check for eof, then call binmode. For example:
-
-  # First line
-  # Second line
-
-  open FH, "$^X -e \"open F, '$0';binmode STDOUT;print <F>\" |";
-
-  eof(FH);
-  binmode(FH);
-
-  print "First line:", scalar <FH>, "\n";
-  print "Second line:", scalar <FH>, "\n";
-
-  close FH;
-
-One solution is to make sure that you only call binmode immediately after
-opening the filehandle. I'm not aware of any workaround for this bug that
-FileHandle::Unget could implement. However, the module does detect this
-situation and prints a warning.
+No known bugs.
 
 Contact david@coppit.org for bug reports and suggestions.
 
